"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Paxcount(google.protobuf.message.Message):
    """
    TODO: REPLACE
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    WIFI_FIELD_NUMBER: builtins.int
    BLE_FIELD_NUMBER: builtins.int
    UPTIME_FIELD_NUMBER: builtins.int
    wifi: builtins.int
    """
    seen Wifi devices
    """
    ble: builtins.int
    """
    Seen BLE devices
    """
    uptime: builtins.int
    """
    Uptime in seconds
    """
    def __init__(
        self,
        *,
        wifi: builtins.int = ...,
        ble: builtins.int = ...,
        uptime: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["ble", b"ble", "uptime", b"uptime", "wifi", b"wifi"]) -> None: ...

global___Paxcount = Paxcount
