# Note: Docs are generated from this command below, albeit from Vercel.
# The docs/ dir is not used and is no longer committed.
# see sachaw if you have questions
pdoc3 --html -f --output-dir docs meshtastic
